"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadNS = void 0;
const ytdl_core_muxer_by_darky_1 = __importDefault(require("ytdl-core-muxer-by-darky"));
const fluent_ffmpeg_1 = __importDefault(require("fluent-ffmpeg"));
const nanostores_cjs_1 = require("nanostores-cjs");
const repl_ns_1 = require("repl-ns");
const duration_1 = require("../src/duration");
const ffmpeg_static_1 = __importDefault(require("ffmpeg-static"));
const p_event_1 = __importDefault(require("p-event"));
const events_1 = require("events");
const ts_pattern_1 = require("ts-pattern");
const fs_1 = require("../src/fs");
const promises_1 = require("stream/promises");
if (ffmpeg_static_1.default)
    process.env['FFMPEG_PATH'] = ffmpeg_static_1.default;
exports.downloadNS = (0, repl_ns_1.ns)('download', {
    async onDownload(ctx) {
        const isBusy = (0, ts_pattern_1.match)((0, exports.downloadNS)().downloadStatus$.get().status)
            .with(ts_pattern_1.P.union('nothing', 'completed', 'error'), () => false)
            .with(ts_pattern_1.P.union('downloading', 'processing'), () => true)
            .exhaustive();
        if (isBusy) {
            return;
        }
        const ac = new AbortController();
        (0, exports.downloadNS)().renderDownloading();
        try {
            const cachedTempFilePath = (0, fs_1.fsNS)().cachedTempFilePath(ctx.url);
            const temporaryFilePath = cachedTempFilePath ?? (await (0, fs_1.fsNS)().createTempFilePath());
            if (!cachedTempFilePath) {
                const ytDownloading = (0, exports.downloadNS)().youtubeDownload(ctx.url);
                process.nextTick(async () => {
                    try {
                        for await (const [, downloadedBytes, totalBytes] of (0, events_1.on)(ytDownloading, 'progress', { signal: ac.signal })) {
                            (0, exports.downloadNS)().renderDownloading(`${(downloadedBytes / 1024 / 1024).toFixed(0)} of ${(totalBytes / 1024 / 1024).toFixed(0)} MB`);
                        }
                    }
                    catch { }
                });
                await (0, promises_1.pipeline)(ytDownloading, (0, fs_1.fsNS)().writeTempFile(temporaryFilePath));
            }
            if (ctx.startTime || ctx.endTime || ctx.resolution !== 'highest') {
                (0, exports.downloadNS)().renderProcessing();
                const ffmpegStream = await Promise.resolve((0, exports.downloadNS)().createFfmpeg(temporaryFilePath))
                    .then(ff => (ctx.startTime ? ff.setStartTime(ctx.startTime) : ff))
                    .then(ff => (ctx.endTime ? ff.setDuration((0, duration_1.durationNS)().calcDuration(ctx)) : ff))
                    .then(ff => (ctx.resolution === 'highest' ? ff : ff.size(`?x${ctx.resolution}`)));
                process.nextTick(async () => {
                    try {
                        for await (const [{ timemark }] of (0, events_1.on)(ffmpegStream, 'progress', { signal: ac.signal })) {
                            (0, exports.downloadNS)().renderProcessing(timemark);
                        }
                    }
                    catch { }
                });
                await (0, p_event_1.default)(ffmpegStream.saveToFile(ctx.path), 'end');
            }
            else {
                await (0, fs_1.fsNS)().cpFile(temporaryFilePath, ctx.path);
            }
            !cachedTempFilePath && (0, fs_1.fsNS)().setCachedTempFilePath(ctx.url, temporaryFilePath);
            (0, exports.downloadNS)().renderComplete();
        }
        catch (err) {
            (0, exports.downloadNS)().renderErr(err);
        }
        finally {
            ac.abort();
        }
    },
    youtubeDownload(url) {
        return (0, ytdl_core_muxer_by_darky_1.default)(url);
    },
    createFfmpeg(path) {
        return (0, fluent_ffmpeg_1.default)(path);
    },
    renderComplete() {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'completed', payload: '' });
    },
    renderErr(err) {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'error', payload: err.message });
    },
    renderDownloading(payload = '') {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'downloading', payload });
    },
    renderProcessing(payload = '') {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'processing', payload });
    },
    downloadStatus$: (0, nanostores_cjs_1.atom)({
        status: 'nothing',
        payload: '',
    }),
});
